void tocRead() // read the table of contents (TOC) from the EEPROM (page 448)
{
  mem_1.readPage(448, rwBuffer);
  for (byte i = 0; i < 14; i++)
  {
    toc[i] = rwBuffer[i];
  }
}

void tocWrite(byte addLoc)
{
  byte tocByte = addLoc/8; // in which byte of the toc array is the location (each byte corresponds to a row of buttons)
  toc[tocByte] = bitSet(toc[tocByte], 7 - (addLoc % 8)); // turns the appropriate byte on (1)
  for (byte i = 0; i < 14; i++) // update the read/write buffer with the current toc
    rwBuffer[i] = toc[i];

  mem_1.writePage(448, rwBuffer); // write the buffer to the toc page on the EEPROM (448)
}

void tocClear(byte clearLoc)
{
  byte tocByte = clearLoc/8; // in which byte of the toc array is the location (each byte corresponds to a row of buttons)
  toc[tocByte] = bitClear(toc[tocByte], 7 - (clearLoc % 8)); // turns the appropriate byte off (0)
  for (byte i = 0; i < 14; i++) // update the read/write buffer with the current toc
    rwBuffer[i] = toc[i];

  mem_1.writePage(448, rwBuffer); // write it to the toc page on the EEPROM (448)
}

boolean checkToc(byte thisLoc) // check whether a memory location is already used or not
{
  byte tocByte = thisLoc/8; // in which byte of the toc array is the location (each byte corresponds to a row of buttons)
  if bitRead(toc[tocByte], 7 - (thisLoc % 8))
    return true;
  else
    return false;  
}

void savePatch(byte patchnumber)
{
  lockPot(6);

  int pageNum = patchnumber * 8; // each patch is 8 pages long


  for (byte i = 0; i < 4; i++)
  {
    rwBuffer[i] = kick[i];
    rwBuffer[i + 4] = snare[i];
    rwBuffer[i + 8] = hat[i];
    rwBuffer[i + 12] = crash[i];
  }
  mem_1.writePage(pageNum, rwBuffer);

  rwBuffer[0] = kickVolume;
  rwBuffer[1] = snareVolume;
  rwBuffer[2] = hatVolume;
  rwBuffer[3] = crashVolume;

  rwBuffer[4] = bpm;
  rwBuffer[5] = swing;

  rwBuffer[6] = hatAutoOn;
  rwBuffer[7] = crashAutoOn;
  rwBuffer[8] = hatPitch;
  rwBuffer[9] = crashPitch;

  mem_1.writePage(pageNum + 1, rwBuffer);

  for (byte i = 0; i < 16; i++)
    rwBuffer[i] = hatAuto[i];

  mem_1.writePage(pageNum + 2, rwBuffer);

  for (byte i = 0; i < 16; i++)
    rwBuffer[i] = hatAuto[i + 16];

  mem_1.writePage(pageNum + 3, rwBuffer);

  for (byte i = 0; i < 16; i++)
    rwBuffer[i] = crashAuto[i];

  mem_1.writePage(pageNum + 4, rwBuffer);

  for (byte i = 0; i < 16; i++)
    rwBuffer[i] = crashAuto[i + 16];

  mem_1.writePage(pageNum + 5, rwBuffer);

}


void loadPatch(byte patchnumber)
{
  lockPot(6);

  int pageNum = patchnumber * 8; // each patch is 8 pages long

  mem_1.readPage(pageNum, rwBuffer);


  for (byte i = 0; i < 4; i++)
  {
    kick[i] = rwBuffer[i];
    snare[i] = rwBuffer[i + 4];
    hat[i] = rwBuffer[i + 8];
    crash[i] = rwBuffer[i + 12];
  }

  mem_1.readPage(pageNum + 1, rwBuffer);

  kickVolume = rwBuffer[0];
  snareVolume = rwBuffer[1];
  hatVolume = rwBuffer[2];
  crashVolume = rwBuffer[3];

  if (bpm != rwBuffer[4])
  {
    bpm = rwBuffer[4];
    Timer1.setPeriod(60000000 / bpm / 96);
  }

  tempSwing = rwBuffer[5];

  hatAutoOn = rwBuffer[6];
  crashAutoOn = rwBuffer[7];
  hatPitch = rwBuffer[8];
  crashPitch = rwBuffer[9];

  mem_1.readPage(pageNum + 2, rwBuffer);

  for (byte i = 0; i < 16; i++)
    hatAuto[i] = rwBuffer[i];

  mem_1.readPage(pageNum + 3, rwBuffer);

  for (byte i = 0; i < 16; i++)
    hatAuto[i + 16] = rwBuffer[i];

  mem_1.readPage(pageNum + 4, rwBuffer);

  for (byte i = 0; i < 16; i++)
    crashAuto[i] = rwBuffer[i];

  mem_1.readPage(pageNum + 5, rwBuffer);

  for (byte i = 0; i < 16; i++)
    crashAuto[i + 16] = rwBuffer[i];
}

void loadCheck()
{
  if (cued != 255 && (currentStep == 0 || currentStep == 16))
  {

    loadPatch(cued);
    nowPlaying = cued;
    slowBlink = roughMillis;
    cued = 255;
  }

}

















